﻿#####################################################################
# Backup-MsftBlog.ps1
# Version 2.2
#
# Downloads Microsoft MSDN and TechNet blog to a local folder as a set
# of HTML pages without Microsoft branding
#
# Vadims Podans (c) 2019
# https://www.sysadmins.lv/
#####################################################################
# v1.0 -- initial release
# v2.0 -- added blog post images download
# v2.1 -- html update bug fix
# v2.2 -- fix for some weird cyrillic post titles
param(
    [Parameter(Mandatory = $true)]
    [Uri]$BlogUri,
    [Parameter(Mandatory = $true)]
    [IO.DirectoryInfo]$OutputDirectory
)

$ErrorActionPreference = "stop"

# dependency load: https://www.nuget.org/packages/HtmlAgilityPack/
Add-Type -Path HtmlAgilityPack.dll
Add-Type -AssemblyName System.Web

Write-Verbose "Starting blog: $BlogUri"
    if (!(Test-Path $OutputDirectory)) {
    md -Path $OutputDirectory -Force | Out-Null
}
function __htmlDocFromUri([Uri]$uri) {
    $wr = Invoke-WebRequest -Uri $uri.AbsoluteUri
    $doc = New-Object htmlagilitypack.htmldocument
    $doc.LoadHtml($wr.Content)
    $doc
}
function __sanitizePostName($name) {
    $name = $name.Trim()
    $name = [Web.HttpUtility]::HtmlDecode($name)
    $name = $name -replace "\\", "" `
        -replace "/", "" `
        -replace ":", "" `
        -replace "\*", "" `
        -replace "\?", "" `
        -replace '"', "" `
        -replace "<", "" `
        -replace ">", "" `
        -replace "\|", "" `
        -replace "\0", ""
    # reduce HTML file name to avoid 
    if ($name.Length -gt 150) {
        $name = $name.Substring(0, 150)
    }
    Write-Verbose "HTML file name: '$name'"
    $name
}
function __getImgFileName([uri]$url) {
    $url.Segments[-1]
}
filter __dlImage {
    # check if img is wrapped in <a> tag.
    if ($_.ParentNode.Name -eq "a") {
        # image may fail if source file is dead. It is not critical and proceed with the next
        # item if image cannot be found.
        try {
            $imgName = ([uri]$_.ParentNode.Attributes['href'].Value).Segments[-1]
            $img = Invoke-WebRequest -Uri $_.ParentNode.Attributes['href'].Value
            $_.ParentNode.Attributes['href'].Value = $imgName
            Set-Content -Path (Join-Path $script:postDir $imgName) -Value $img.Content -Encoding byte
        } catch { }
    }
    try {
        $imgName = ([uri]$_.Attributes['src'].Value).Segments[-1]
        $img = Invoke-WebRequest -Uri $_.Attributes['src'].Value
        # if image with the same name exists, add thumbnail prefix to file name
        if ((Test-Path (Join-Path $script:postDir $imgName))) {
            $imgName = "thumb_" + $imgName
        }
        $_.Attributes['src'].Value = $imgName
        Set-Content -LiteralPath (Join-Path $script:postDir $imgName) -Value $img.Content -Encoding byte
    } catch { }
}
function __processImages($postDoc) {
    $imgTags = $postDoc.SelectNodes("//img[@src]")
    if (!$imgTags) {
        return
    }
    $imgTags | __dlImage
}
function __savePost($blogPost) {
    Set-Content -LiteralPath (Join-Path $script:postDir ($script:postSanitizedName + ".html")) `
        -Value $blogPost.HtmlContent
}
filter __parseBlogPost() {
    $postDoc = __htmlDocFromUri $_.Attributes['href'].Value
    $blogPost = New-Object psobject -Property @{
        'URL' = $_.Attributes['href'].Value;
        'PostTitle' = $postDoc.DocumentNode.SelectSingleNode("//h1[@class='entry-title']").InnerText;
        'PostAuthor' = $postDoc.DocumentNode.SelectSingleNode("//a[@data-profile-userid]").InnerText;
        'PostDate' = $postDoc.DocumentNode.SelectSingleNode("//time[contains(@class,'entry-date')]").Attributes['datetime'].Value
        'HtmlContent' = ""
    }
    Write-Verbose "Parsing blog post: $($blogPost.PostTitle)"
    $dt = Get-Date $blogPost.PostDate
    $script:postSanitizedName = __sanitizePostName $blogPost.PostTitle
    $script:postDir = md (Join-Path $OutputDirectory ($dt.ToString("yyyy.MM.dd-HHmmss"))) -Force
    # HtmlAgilityPack makes search within entire document. We need to find images only within the post
    $postNode = New-Object htmlagilitypack.htmldocument
    $postNode.LoadHtml($postDoc.DocumentNode.SelectSingleNode("//div[contains(@class,'entry-content')]").OuterHtml)
    __processImages $postNode.DocumentNode
    $blogPost.HtmlContent = @"
<!DOCTYPE html>
<html>
<head>
<link rel="stylesheet" href="../../style.css" type="text/css" media="all" />
</head>
<body>
<div class="post">
    <div class="title"><h1>$($blogPost.PostTitle)</h1></div>
    <div class="meta">$($blogPost.PostAuthor) | <a href="$($blogPost.URL)">$($blogPost.PostDate)</a></div>
	<hr />
    $($postNode.DocumentNode.OuterHtml)
</div>
</body>
</html>
"@
    __savePost $blogPost 
}
filter __parsePagerPage() {
    Write-Verbose "Processing page: $_"
    $pagerDoc = __htmlDocFromUri ($BlogUri.AbsoluteUri + "page/$_/")
    $posts = $pagerDoc.DocumentNode.SelectNodes("//a[@rel='bookmark']")
    if (!$posts) {
        return
    }
    # iterate over posts on a current pagination page
    $posts | __parseBlogPost
}
$doc = __htmlDocFromUri $BlogUri
# get pager element at the bottom of the page to determine the number of pages in the blog.
# We will iterate over pages to download posts.
$mainPager = $doc.DocumentNode.SelectSingleNode("//div[@class='nav-links']")
$nodeCount = $mainPager.ChildNodes.Count
# get the number of pages in the pager
try {
    [int]$pageCount = $mainPager.ChildNodes[$nodeCount - 3].InnerText
} catch {
    [int]$pageCount = $mainPager.ChildNodes[$nodeCount - 4].InnerText
}
Write-Verbose "Number of pages (10 posts per page): $pageCount"
# iterate over pagination pages
1..$pageCount | __parsePagerPage